import debug from "debug";
import { Etcd3, IOptions } from "etcd3";
import { load } from "js-yaml";
import { hostname } from "os";
import { Options } from "../interfaces/options.interface";
import { getAppPkg } from "../utils/app-pkg";
import { BaseConfiguration } from "./base-configuration.interface";

const configKeyPrefix = "share/config";
const logger = debug("fennec:config:loader:etcd");

export async function readEtcdConfig<T = BaseConfiguration>({
  etcd,
  appName,
  environment,
}: Options): Promise<T> {
  if (!(etcd instanceof Etcd3)) {
    etcd = new Etcd3(etcd);
  }

  if (appName == undefined) {
    appName = await getAppPkg().then((pkg) => pkg.name);
    debug.log("found appName from package.json. [%s]", appName);
  }

  const key = `${configKeyPrefix}/${environment ?? hostname()}/${appName}`;
  debug.log('etcd key: "%s"', key);

  const plainText = await etcd.get(key);

  if (plainText === null) {
    debug.log('No configuration found at "%s"', key);
    throw new Error(`No configuration found at ${key}`);
  }
  try {
    return (await load(plainText)) as T;
  } catch (error) {
    logger.log("Can not parse value as yaml");
    logger.log("plainText: %s", plainText);
    logger.log("%O", error);
    throw new Error(`Failed to parse configuration at ${key}`);
  }
}
