import debug from "debug";
import { readFile } from "fs/promises";
import { load } from "js-yaml";
import { Options } from "../interfaces/options.interface";
import { findUp } from "../utils/find-up";
import { BaseConfiguration } from "./base-configuration.interface";

const configKeyPrefix = "share/config";
const logger = debug("fennec:config:loader:etcd");

export async function readFileConfig<
  T = BaseConfiguration
>({}: Options = {}): Promise<T> {
  const path = await findUp(["config.yml", "config.yaml"], process.cwd());
  try {
    if (path) {
      return readFile(path, "utf8").then((str) => load(str) as T);
    } else {
      throw new Error("No config file found");
    }
  } catch (error) {
    logger.log("Can not parse file as yaml");
    logger.log("File path: %s", path);
    logger.log("%O", error);
    throw new Error(`Failed to parse configuration at ${path}`);
  }
}
