import path from "path";
import fs from "fs/promises";

export const findUp = async (
  filenameList: string[] | string,
  from: string
): Promise<string | null> => {
  if (!Array.isArray(filenameList)) {
    filenameList = [filenameList];
  }
  try {
    const isDir = await fs.stat(from).then((stat) => stat.isDirectory());
    if (!isDir) {
      return null;
    }
    for (const filename of filenameList) {
      const filepath = path.resolve(from, filename);
      if (
        await fs
          .stat(filepath)
          .then((stat) => stat.isFile() || stat.isDirectory())
      ) {
        return filepath;
      }
    }
  } catch (err: any) {
    if (err.code === "ENOENT") {
      return null;
    }
    throw err;
  }
  return findUp(filenameList, path.resolve(from, ".."));
};
